package com.example.civilizedcity.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.*;
import com.example.civilizedcity.mapper.TaFeedbackMapper;
import com.example.civilizedcity.mapper.TaIssueMapper;
import com.example.civilizedcity.mapper.TaIssueProcessMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.mapper.TaOrgIssueMapper;
import com.example.civilizedcity.service.TaOrgIssueService;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * 单位问题单;(ta_org_issue)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2022-12-28
 */
@Service
public class TaOrgIssueServiceImpl extends BaseServiceImpl<TaOrgIssueMapper, TaOrgIssue> implements TaOrgIssueService {

    @Autowired
    TaIssueMapper taIssueMapper;

    @Autowired
    TaIssueProcessMapper taIssueProcessMapper;

    @Autowired
    TaFeedbackMapper taFeedbackMapper;

    @Override
    public IPage<TaIssue> getIssuePageBy(IPage<TaIssue> pg, String orgId, String sourceType, String bizStatus) {

        String processNode = null;
        Boolean isExpire = false;

        if (!StringUtils.isEmpty(bizStatus)) {
            if ("expire".equals(bizStatus)) {
                isExpire = true;
            } else {
                processNode = bizStatus;
            }
        }

        return baseMapper.getIssuePageBy(pg, orgId, sourceType, processNode, isExpire);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void createNewIssue(TaIssue taIssue, TaIssueProcess taIssueProcess, SysUser sysUser) throws Exception {
        if (Constants.SOURCE_FEEDBACK.equals(taIssue.getSourceType())) {
            // 如果是市民提的问题单
            TaFeedback taFeedback = taFeedbackMapper.selectById(taIssue.getSourceTarget());
            if (null == taFeedback) {
                throw new Exception("未找到市民随手拍记录");
            }

            taFeedback.setVerifyDate(LocalDateTime.now());
            taFeedback.setVerifyStatus(Constants.APPLY_PASS);
            taFeedbackMapper.updateById(taFeedback);
        }

        // 1. 先修改主记录信息
        taIssue.setProcessNode(Constants.PROCESS_ASSIGNED);
        taIssue.setProcessStatus(Constants.APPLY_PASS);
        taIssue.setOrgId(taIssueProcess.getNextOrg());
        taIssue.setExpireDate(taIssueProcess.getExpireDate());
        taIssue.setProcessNum(taIssue.getProcessNum() + 1); // 交办次数
        taIssueMapper.updateById(taIssue);

        // 2. 新增机构问题单
        TaOrgIssue taOrgIssue = new TaOrgIssue();
        taOrgIssue.setIssueId(taIssue.getIssueId());
        taOrgIssue.setOrgId(taIssue.getOrgId());
        taOrgIssue.setProcessNode(Constants.PROCESS_START);
        taOrgIssue.setProcessStatus(Constants.APPLY_READY); // 状态是未审核
        taOrgIssue.setStatus(Constants.STATUS_NORMAL);
        taOrgIssue.setCreateDate(LocalDateTime.now());
        taOrgIssue.setCreateUser(sysUser.getUserId());
        save(taOrgIssue);

        // 3. 保存流程节点记录
        taIssueProcess.setProcessId(null);
        taIssueProcess.setProcessNode(Constants.PROCESS_ASSIGNED);
        taIssueProcess.setProcessStatus(Constants.APPLY_PASS);
        taIssueProcess.setCreateUser(sysUser.getUserId());
        taIssueProcess.setCreateDate(LocalDateTime.now());
        taIssueProcess.setStatus(Constants.STATUS_NORMAL);
        taIssueProcessMapper.insert(taIssueProcess);
    }

    @Override
    public TaOrgIssue getByIssueAndOrg(Integer issueId, String orgId) {
        return baseMapper.getByIssueAndOrg(issueId, orgId);
    }
}